{% extends "layout.html" %}
{% block body %}

<h1>{{ _('Build HAT Documentation') }}</h1>

<p>{{ _('Welcome! This is the documentation for the Raspberry Pi Build HAT') }}{{ release|e }}{% if last_updated %}, {{ _('last updated') }} {{ last_updated|e }}{% endif %}.</p>

<table class="contentstable">
  <tr>
    <td width="40%" style="padding-left:2em;">
      <p class="biglink">
        <a class="biglink" href="{{ pathto("buildhat/index") }}">Library Reference</a><br/>
          <span class="linkdescr">Library</a></span>
      </p>
    </td>
    <td width="40%" style="padding-left:2em;">
      <p class="biglink">
        <a class="biglink" href="https://www.raspberrypi.com/documentation/accessories/build-hat.html">Getting Started</a><br/>
          <span class="linkdescr">Getting Started guide</a></span>
      </p>
    </td>
    <td width="40%" style="padding-left:2em;">
      <p class="biglink">
        <a class="biglink" href="{{ pathto("license") }}">License</a><br/>
        <span class="linkdescr">License information</span>
      </p>
    </td>
  </tr>
</table>

{% endblock %}
