<div class="rst-versions" data-toggle="rst-versions" role="note" aria-label="versions">
  <span class="rst-current-version" data-toggle="rst-current-version">
    <span class="fa fa-book"> Ports and Versions</span>
    {{ port }} ({{ port_version }})
    <span class="fa fa-caret-down"></span>
  </span>
  <div class="rst-other-versions">
    <dl>
      <dt>Ports</dt>
      {% for slug, url in all_ports %}
        <dd><a href="{{ url }}">{{ slug }}</a></dd>
      {% endfor %}
    </dl>
    <dl>
      <dt>Versions</dt>
      {% for slug, url in all_versions %}
        <dd><a href="{{ url }}">{{ slug }}</a></dd>
      {% endfor %}
    </dl>
    <dl>
      <dt>Downloads</dt>
      {% for type, url in downloads %}
        <dd><a href="{{ url }}">{{ type }}</a></dd>
      {% endfor %}
    </dl>
    <hr/>
    <dl>
      <dt>Links</dt>
      <dd>
        <a href="https://github.com/RaspberryPiFoundation/python-build-hat">GitHub</a>
      </dd>
    </dl>
  </div>
</div>
